function createContentElement($textarea) {
    $editableArea = $textarea.parent();
    var marginPaddingAndBorder = $editableArea.outerHeight(true) - $editableArea.height();
    $editableArea.height($editableArea.parent().height() - $editableArea.prev().outerHeight(true) - marginPaddingAndBorder);

    var iframe = $('<iframe />', {
                        src: 'javascript:"<html></html>"',
                        frameBorder: '0'
                    })
                    .css({
                    width: '100%',
                    height: '100%'
                    })
                    .insertBefore($textarea)[0];

    var window = iframe.contentWindow || iframe;
    var document = window.document || iframe.contentDocument;
        
    // <img>\s+\w+ creates invalid nodes after cut in IE
    var html = $textarea.val().replace(/(<\/?img[^>]*>)[\r\n\v\f\t ]+/ig, '$1');

    document.designMode = 'On';
    document.open();
    document.write(
        '<!DOCTYPE><html><head xmlns="http://www.w3.org/1999/xhtml">' +
        '<meta http-equiv="Content-Type" content="text/html; charset=UTF-8" />' +
        '<style type="text/css">' +
            'html,body{padding:0px;margin:0;font:12px/1.4em Verdana,Geneva,sans-serif;background:#fff;}' + 
            'p{margin:.7em 0;}p:first-child{margin-top:0;}.t-marker{display:none;}' + 
            'a{color:#00a}' +
        '</style>' +
        '</head><body spellcheck="false">' +
        html +
        '</body></html>');
        
    document.close();

    return window;
};

function selectionChanged(editor) {
    $t.trigger(editor.element, 'selectionChange');
}

function removePendingFormats(editor) {
    if (editor.pendingFormats.length == 0) return;

    editor.pendingFormats.reverse();

    $.each(editor.pendingFormats, function() {
        for (var node = this.firstChild; node; node = node.nextSibling)
            while (node.nodeType == 3 && (charIndex = node.nodeValue.indexOf('\ufeff')) >= 0)
                node.deleteData(charIndex, 1);
    });

    $.each(editor.pendingFormats, function() {
        if (this.innerHTML == '' && this.parentNode)
            dom.remove(this);
    });

    editor.pendingFormats = [];
}

$t.editor = function (element, options) {
    /* suppress initialization in mobile webkit devices (w/o proper contenteditable support) */
    if (/Mobile.*Safari/.test(navigator.userAgent))
        return;

    var self = this;

    this.element = element;

    var $element = $(element);

    if ($element.is('textarea'))
        $t.editor.extend(element);

    $element.closest('form').bind('submit', function () {
        self.update();
    });

    $.extend(this, options);

    $t.bind(this, {
        load: this.onLoad,
        selectionChange: this.onSelectionChange,
        change: this.onChange,
        execute: this.onExecute
    });
        
    this.textarea = $element.find('textarea').attr('autocomplete', 'off')[0];
    this.window = createContentElement($(this.textarea));
    this.document = this.window.contentDocument || this.window.document;
    this.body = this.document.body;
    this.keyboard = new Keyboard([new TypingHandler(this), new SystemHandler(this)]);
        
    this.clipboard = new Clipboard(this);

    this.pendingFormats = [];
        
    var buttons = '.t-editor-button .t-icon';

    this.undoRedoStack = new UndoRedoStack();

    function toolFromClassName(element) {
        return $.grep(element.className.split(' '), function (x) {
            return !/^t-(icon|state-hover|header|combobox|dropdown)$/i.test(x);
        })[0].substring(2).toLowerCase();
    }

    function descriptionText(tool) {
        if (!tool.key)
            return tool.description;

        return new $t.stringBuilder()
            .cat(tool.description)
            .cat(' (').catIf('Ctrl + ', tool.ctrl).catIf('Shift + ', tool.shift).catIf('Alt + ', tool.alt).cat(tool.key).cat(')')
            .string();
    }

    $(this.window).bind('blur', function () {
        var old = self.textarea.value;
        var value = self.encodedValue();
        self.update(value);
        if (value != old)
            $t.trigger(self.element, 'change')
    });

    var toolbarItems = '.t-editor-toolbar > li > *';

    var selectionRestorePoint;

    $element
        .delegate(buttons, 'mouseenter', $t.hover)
        .delegate(buttons, 'mouseleave', $t.leave)
        .delegate(buttons, 'mousedown', $t.preventDefault)
        .delegate(buttons, 'click', $t.stopAll(function (e) {
            self.focus();
            self.exec(toolFromClassName(this));
        }))
        .find(toolbarItems)
            .each(function () {
                var $this = $(this);
                var tool = self.tools[toolFromClassName(this)];
                var description = descriptionText(tool);

                if ($this.is('a.t-icon'))
                    $this.attr({ unselectable: 'on', title: description });
                else if ($this.is('.t-combobox') && tool)
                    $this
                        .find('.t-input').attr({ title: description }).bind('keydown', function(e) {
                            if (e.keyCode == 13) {
                                e.preventDefault();
                                var combo = $this.data('tComboBox')
                                    .value($(this).val());
                            }
                        }).end()
                        .tComboBox($.extend({
                            onChange: function(e) {
                                self.focus();
                                if (selectionRestorePoint) {
                                    self.selectRange(selectionRestorePoint.toRange());
                                    self.exec(toolFromClassName(this), { value: e.value });
                                }
                            }
                        }, tool));
                else if ($this.is('.t-dropdown') && tool)
                    $this.attr({ title: description })
                        .find('*').attr('unselectable', 'on').end()
                        .tDropDownList($.extend({
                            onChange: function(e) {
                                self.exec(toolFromClassName(this), { value: e.value });
                            }
                        }, tool));
            }).end()
        .bind('selectionChange', function() {
            var range = self.getRange();
            var nodes = textNodes(range);
            if (!nodes.length)
                nodes = [range.startContainer];

            $element
                .find(toolbarItems).each(function () {
                    var $this = $(this);
                    var tool = self.tools[toolFromClassName(this)];
                        
                    if ($this.is('a.t-icon'))
                        $this.toggleClass('t-state-active', tool.finder().isFormatted(nodes));
                    else if ($this.is('.t-combobox') && tool) {
                        var list = $this.data('tComboBox');
                        list.close();
                        list.text(tool.finder().getFormat(nodes).split(',')[0].replace(/'/g, ''));
                        list.select(-1);
                    } else if ($this.is('.t-dropdown') && tool) {
                        var list = $this.data('tDropDownList');
                        list.close();
                        var format = tool.finder().getFormat(nodes);
                        for (var i = 0; i < list.data.length; i++)
                            if (list.data[i].Value.indexOf(format) >= 0) {
                                list.select(i);
                                break;
                            }
                    }
                });
        });

    $(this.document)
        .bind('keyup', function (e) {
            var selectionCodes = [8, 9, 13, 33, 34, 35, 36, 37, 38, 39, 40, 40, 45, 46];

            if ($.inArray(e.keyCode, selectionCodes) > -1)
                selectionChanged(self);
        })
        .keydown(function (e) {
            var toolName = self.keyboard.toolFromShortcut(self.tools, e);
            if (toolName) {
                e.preventDefault();
                self.exec(toolName);
                return false;
            }

            self.keyboard.clearTimeout();

            self.keyboard.keydown(e);
        })
        .keyup(function (e) {
            self.keyboard.keyup(e);
        })
        .mouseup(function () {
            selectionChanged(self);
        });
    
    $(this.body)
        .bind('cut paste', function (e) {
            self.clipboard['on' + e.type](e);
        });

    $(this.element.ownerDocument)
        .bind('mousedown', function() {
            var newRange;

            try { newRange = self.getRange(); } catch (ex) { }

            if (newRange && newRange.startContainer.ownerDocument == self.document)
                selectionRestorePoint = new RestorePoint(self.getRange());
        })

};

$.extend($t.editor, {
    Dom: dom,
    RestorePoint: RestorePoint,
    Marker: Marker,
    RangeUtils: RangeUtils,
    RangeEnumerator: RangeEnumerator,
    LinkFormatter: LinkFormatter,
    LinkFormatFinder: LinkFormatFinder,
    LinkCommand: LinkCommand,
    UnlinkCommand: UnlinkCommand,
    InlineFormatter: InlineFormatter,
    InlineFormatFinder: InlineFormatFinder,
    BlockFormatter: BlockFormatter,
    GreedyBlockFormatter: GreedyBlockFormatter,
    BlockFormatFinder: BlockFormatFinder,
    FormatCommand: FormatCommand,
    ListFormatFinder: ListFormatFinder,
    ListFormatter: ListFormatter,
    ListCommand: ListCommand,
    ParagraphCommand: ParagraphCommand,
    NewLineCommand: NewLineCommand,
    ImageCommand: ImageCommand,
    GenericCommand: GenericCommand,
    UndoRedoStack: UndoRedoStack,
    TypingHandler: TypingHandler,
    SystemHandler: SystemHandler,
    Keyboard: Keyboard,
    MSWordFormatCleaner: MSWordFormatCleaner
});

// public api
$t.editor.prototype = {
    value: function (html) {
        if (html === undefined) return domToXhtml(this.body);

        // Some browsers do not allow setting CDATA sections through innerHTML so we encode them as comments
        html = html.replace(/<!\[CDATA\[(.*)?\]\]>/g, '<!--[CDATA[$1]]-->');

        // Encode script tags to avoid execution and lost content (IE)
        html = html.replace(/<script([^>]*)>(.*)?<\/script>/ig, '<telerik:script $1>$2<\/telerik:script>');

        if ($.browser.msie) {
            // Internet Explorer removes comments from the beginning of the html
            html = '<br/>' + html;

            // IE < 8 makes href and src attributes absolute
            html = html.replace(/href\s*=\s*(?:'|")?([^'">\s]*)(?:'|")?/, 'originalhref="$1"');
            html = html.replace(/src\s*=\s*(?:'|")?([^'">\s]*)(?:'|")?/, 'originalsrc="$1"');

            this.body.innerHTML = html;
            dom.remove(this.body.firstChild);

            $('script,link,img,a', this.body).each(function () {
                if (this['originalhref']) {
                    this.setAttribute('href', this['originalhref']);
                    this.removeAttribute('originalhref');
                }
                if (this['originalsrc']) {
                    this.setAttribute('src', this['originalsrc']);
                    this.removeAttribute('originalsrc');
                }
            })
        } else {
            this.body.innerHTML = html;
        }

        this.update();
    },

    focus: function () {
        this.window.focus();
    },

    update: function (value) {
        this.textarea.value = value || this.isEncoded ? this.encodedValue() : this.value();
    },

    encodedValue: function () {
        return dom.encode(this.value());
    },

    createRange: function (document) {
        return createRange(document || this.document);
    },

    getSelection: function () {
        return selectionFromDocument(this.document);
    },
        
    selectRange: function(range) {
        var selection = this.getSelection();
        selection.removeAllRanges();
        selection.addRange(range);
    },

    getRange: function () {
        var selection = this.getSelection();
        return selection.rangeCount > 0 ? selection.getRangeAt(0) : this.createRange();
    },

    exec: function (name, params) {
        var tool = this.tools[name.toLowerCase()];

        if (tool) {
            var range = this.getRange();
            var command = tool.command ? tool.command($.extend({ range: range }, params)) : null;

            $t.trigger(this.element, 'execute', { name: name, command: command });

            if (/undo|redo/i.test(name)) {
                this.undoRedoStack[name.toLowerCase()]();
            } else {
                this.undoRedoStack.push(command);

                command.editor = this;
                command.exec();

                if (command.async) {
                    command.change = $.proxy(function () { selectionChanged(this); }, this);
                    return;
                }
            }

            selectionChanged(this);
        }
    }
}

$.fn.tEditor = function (options) {
    return $t.create(this, {
        name: 'tEditor',
        init: function (element, options) {
            return new $t.editor(element, options);
        },
        options: options
    });
}

var formats = {
    bold: [
        { tags: ['strong'] },
        { tags: ['span'], attr: { style: { fontWeight: 'bold'}} }
    ],

    italic: [
        { tags: ['em'] },
        { tags: ['span'], attr: { style: { fontStyle: 'italic'}} }
    ],

    underline: [{ tags: ['span'], attr: { style: { textDecoration: 'underline'}}}],

    strikeThrough: [
        { tags: ['del'] },
        { tags: ['span'], attr: { style: { textDecoration: 'line-through'}} }
    ],

    justifyLeft: [
        { tags: blockElements, attr: { style: { textAlign: 'left'}} },
        { tags: ['img'], attr: { style: { 'float': 'left'}} }
    ],

    justifyCenter: [
        { tags: blockElements, attr: { style: { textAlign: 'center'}} },
        { tags: ['img'], attr: { style: { display: 'block', marginLeft: 'auto', marginRight: 'auto'}} }
    ],

    justifyRight: [
        { tags: blockElements, attr: { style: { textAlign: 'right'}} },
        { tags: ['img'], attr: { style: { 'float': 'right'}} }
    ],

    justifyFull: [
        { tags: blockElements, attr: { style: { textAlign: 'justify'}} }
    ]
};

function createFormat(shortcut, format, type) {
    return $.extend(shortcut, {
        command: function (options) {
            return new FormatCommand($.extend(options, {
                format: format,
                formatter: function () {
                    return new $t.editor[type + 'Formatter'](format);
                }
            }))
        },
        finder: function () {
            return new $t.editor[type + 'FormatFinder'](format);
        }
    });
}

function inlineFormat(shortcut, format) {
    return createFormat(shortcut, format, 'Inline');
}

function blockFormat(shortcut, format) {
    return createFormat(shortcut, format, 'Block');
}

function formatByName(name, format) {
    for (var i = 0; i < format.length; i++)
        if ($.inArray(name, format[i].tags) >= 0)
            return format[i];
}

$.fn.tEditor.defaults = {
    plugins: [],
    localization: {},
    formats: formats,
    isEncoded: true,
    tools: {
        bold: inlineFormat({ key: 'B', ctrl: true, description: 'Bold' }, formats.bold),
        italic: inlineFormat({ key: 'I', ctrl: true, description: 'Italic' }, formats.italic),
        underline: inlineFormat({ key: 'U', ctrl: true, description: 'Underline' }, formats.underline),
        strikethrough: inlineFormat({ description: 'Strikethrough' }, formats.strikeThrough),
        undo: { key: 'Z', ctrl: true },
        redo: { key: 'Y', ctrl: true },
        insertlinebreak: { key: 13, shift: true, command: function (options) { return new NewLineCommand(options) } },
        insertparagraph: { key: 13, command: function (options) { return new ParagraphCommand(options) } },
        justifycenter: blockFormat({ description: 'Center text' }, formats.justifyCenter),
        justifyleft: blockFormat({ description: 'Align text left' }, formats.justifyLeft),
        justifyright: blockFormat({ description: 'Align text right' }, formats.justifyRight),
        justifyfull: blockFormat({ description: 'Justify' }, formats.justifyFull),
        insertunorderedlist: { description: 'Insert unordered list', finder: function () { return new ListFormatFinder('ul') }, command: function (options) { return new ListCommand($.extend(options, { tag: 'ul' })) } },
        insertorderedlist: { description: 'Insert ordered list', finder: function () { return new ListFormatFinder('ol') }, command: function (options) { return new ListCommand($.extend(options, { tag: 'ol' })) } },
        createlink: { key: 'K', ctrl: true, description: 'Insert hyperlink', finder: function () { return { isFormatted: function () { return false } } }, command: function (options) { return new LinkCommand(options) } },
        unlink: { key: 'K', ctrl: true, shift: true, description: 'Remove hyperlink', finder: function () { return { isFormatted: function () { return false } } }, command: function (options) { return new UnlinkCommand(options) } },
        insertimage: { description: 'Insert image', finder: function () { return { isFormatted: function () { return false } } }, command: function (options) { return new ImageCommand(options) } },
        inserthtml: { description: 'Insert HTML', finder: function () { return new HtmlSnippetFinder() }, command: function(options) { return new InsertHTMLCommand(options); } },
        fontfamily: {
            description: 'Select font family',
            data: [
                { Text: 'Arial', Value: "Arial,Helvetica,sans-serif" },
                { Text: 'Courier New', Value: "'Courier New',Courier,monospace" },
                { Text: 'Georgia', Value: "Georgia,serif" },
                { Text: 'Impact', Value: "Impact,Charcoal,sans-serif" },
                { Text: 'Lucida Console', Value: "'Lucida Console',Monaco,monospace" },
                { Text: 'Tahoma', Value: "Tahoma,Geneva,sans-serif" },
                { Text: 'Times New Roman', Value: "'Times New Roman',Times,serif" },
                { Text: 'Trebuchet MS', Value: "'Trebuchet MS',Helvetica,sans-serif" },
                { Text: 'Verdana', Value: "Verdana,Geneva,sans-serif" }
            ],
            onItemDataBound: function(e) {
                e.text = '<span unselectable="on" style="font: 1.1em ' + e.dataItem.Value + '">' + e.text + '</span>';
            },
            highlightFirst: false,
            finder: function() { return new InlineFormatFinder([{ tags: ['span'] }], 'font-family') },
            command: function (options) {
                return new FormatCommand($.extend(options, {
                    formatter: function () { return new InlineFormatter([{ tags: ['span'] }], { style: { fontFamily: options.value } }, true); }
                }))
            }
        },
        fontsize: {
            description: 'Select font size',
            data: $.map('8,9,10,11,12,14,16,18,20,24,36,48,72'.split(','), function(x) { return { Text: x + 'pt', Value: x } }),
            onItemDataBound: function(e) {
                e.text = '<span unselectable="on" style="font-size: ' + e.dataItem.Text + '">' + e.text + '</span>';
            },
            highlightFirst: false,
            finder: function() { return new InlineFormatFinder([{ tags: ['span'] }], 'font-size') },
            command: function (options) {
                return new FormatCommand($.extend(options, {
                    formatter: function () { return new InlineFormatter([{ tags: ['span'] }], { style: { fontSize: options.value } }, true); }
                }))
            }
        },
        blockformats: {
            description: 'Select block type',
            data: [
                { Text: 'Paragraph', Value: 'p' },
                { Text: 'Quotation', Value: 'blockquote' },
                { Text: 'Address', Value: 'address' },
                { Text: 'Heading 1', Value: 'h1' },
                { Text: 'Heading 2', Value: 'h2' },
                { Text: 'Heading 3', Value: 'h3' },
                { Text: 'Heading 4', Value: 'h4' },
                { Text: 'Heading 5', Value: 'h5' },
                { Text: 'Heading 6', Value: 'h6' }
            ],
            onItemDataBound: function(e) {
                e.text = ['<', ' unselectable="on">' + e.text + '</', '>'].join(e.dataItem.Value);
            },
            finder: function() { return new BlockFormatFinder([{ tags: blockElements }]) },
            command: function (options) {
                return new FormatCommand($.extend(options, {
                    formatter: function () { return new GreedyBlockFormatter([{ tags: [options.value] }], {}); }
                }))
            }
        }
    }
}

})(jQuery);