function InlineFormatFinder(format, greedyProperty) {
    function numberOfSiblings(referenceNode) {
        var count = 0;

        for (var node = referenceNode.parentNode.firstChild; node; node = node.nextSibling)
            if (node != referenceNode && node.className != 't-marker' && node.nodeType == 1)
                count++;

        return count;
    }

    this.findSuitable = function (sourceNode, skip) {
        if (!skip && numberOfSiblings(sourceNode) > 0)
            return null;

        return dom.parentOfType(sourceNode, format[0].tags);
    }

    this.findFormat = function (sourceNode) {
        for (var i = 0; i < format.length; i++) {
            var node = sourceNode;
            var tags = format[i].tags;
            var attributes = format[i].attr;

            if (node && dom.ofType(node, tags) && attrEquals(node, attributes))
                return node;

            while (node) {
                node = dom.parentOfType(node, tags);
                if (node && attrEquals(node, attributes))
                    return node;
            }
        }

        return null;
    }

    if (greedyProperty) {
        this.getFormat = function (nodes) {
            var result = $(isDataNode(nodes[0]) ? nodes[0].parentNode : nodes[0]).css(greedyProperty);

            for (var i = 1, len = nodes.length; i < len; i++)
                if (result != $(isDataNode(nodes[i]) ? nodes[i].parentNode : nodes[i]).css(greedyProperty))
                    return '';

            return result;
        }

        this.isFormatted = function (nodes) {
            return this.getFormat(nodes) !== '';
        }
    } else {
        this.isFormatted = function (nodes) {
            for (var i = 0; i < nodes.length; i++)
                if (this.findFormat(nodes[i]))
                    return true;

            return false;
        }
    }
}

function BlockFormatFinder(format) {
    function contains(node, children) {
        for (var i = 0; i < children.length; i++) {
            var child = children[i];
            if (child == null || !isAncestorOrSelf(node, child))
                return false;
        }

        return true;
    }

    this.findSuitable = function (nodes) {
        var suitable = [];

        for (var i = 0; i < nodes.length; i++)
            suitable.push(dom.parentOfType(nodes[i], format[0].tags));

        for (var i = 0; i < suitable.length; i++)
            if (contains(suitable[i], suitable))
                return suitable[i];

        return null;
    }

    this.findFormat = function (sourceNode) {
        for (var i = 0; i < format.length; i++) {
            var node = sourceNode;
            var tags = format[i].tags;
            var attributes = format[i].attr;

            while (node) {
                if (dom.ofType(node, tags) && attrEquals(node, attributes))
                    return node;
                node = node.parentNode;
            }
        }
        return null;
    }

    this.getFormat = function (nodes) {
        var result = this.findFormat(isDataNode(nodes[0]) ? nodes[0].parentNode : nodes[0]);

        if (!result)
            return '';

        for (var i = 1, len = nodes.length; i < len; i++)
            if (result != this.findFormat(isDataNode(nodes[i]) ? nodes[i].parentNode : nodes[i]))
                return '';

        return result.nodeName.toLowerCase();
    }

    this.isFormatted = function (nodes) {
        for (var i = 0; i < nodes.length; i++)
            if (!this.findFormat(nodes[i]))
                return false;

        return true;
    }
}

function InlineFormatter(format, values, greedyProperty) {
    this.finder = new InlineFormatFinder(format, greedyProperty);

    var attributes = $.extend({}, format[0].attr, values);

    var tag = format[0].tags[0];

    function wrap(node) {
        return dom.wrap(node, dom.create(node.ownerDocument, tag, attributes));
    }

    this.toggle = function (range) {
        var nodes = textNodes(range);
        var isPending = nodes.length == 0;
        var caretMarker;
            
        if (isPending) {
            var markers = getMarkers(range);
            caretMarker = markers[1];
            var invisibleNode = this.editor.document.createTextNode('\ufeff');
            dom.insertBefore(invisibleNode, caretMarker);
            nodes.push(invisibleNode);
            invisibleNode = invisibleNode.cloneNode(true);
            dom.insertAfter(invisibleNode, caretMarker);
            nodes.push(invisibleNode);
            range.setEndAfter(markers[2]);
        }
            
        if (greedyProperty) {
            this.split(range);
            this.apply(nodes);
        } else if (this.finder.isFormatted(nodes)) {
            this.split(range);
            this.remove(nodes);
        } else {
            this.apply(nodes);
        }

        if (isPending)
            this.editor.pendingFormats.push(caretMarker.parentNode);
    }

    this.apply = function (nodes) {
        var formatNodes = [];
        for (var i = 0, l = nodes.length; i < l; i++) {
            var node = nodes[i];

            var formatNode = this.finder.findSuitable(node);
            if (formatNode)
                dom.attr(formatNode, attributes);
            else
                formatNode = wrap(node);

            formatNodes.push(formatNode);
        }

        this.consolidate(formatNodes);
    }

    this.remove = function (nodes) {
        for (var i = 0, l = nodes.length; i < l; i++) {
            var formatNode = this.finder.findFormat(nodes[i]);
            if (formatNode)
                dom.unwrap(formatNode);
        }
    }

    this.split = function (range) {
        var nodes = textNodes(range);

        var isPending = nodes.length == 0;
        var caretMarker;
            
        if (isPending) {
            caretMarker = getMarkers(range)[1];
            nodes.push(caretMarker);
        }

        for (var i = 0, l = nodes.length; i < l; i++) {
            var formatNode = this.finder.findFormat(nodes[i]);
            if (formatNode)
                split(range, formatNode, true);
        }
    }

    this.consolidate = function (nodes) {
        while (nodes.length > 1) {
            var node = nodes.pop();
            var last = nodes[nodes.length - 1];

            if (node.previousSibling && node.previousSibling.className == 't-marker') {
                last.appendChild(node.previousSibling);
            }

            if (node.previousSibling == last) {
                while (node.firstChild)
                    last.appendChild(node.firstChild);
                dom.remove(node);
            }
        }
    }
}

function BlockFormatter(format, values) {
    var finder = new BlockFormatFinder(format);

    function wrap(tag, attributes, nodes) {
        var commonAncestor = nodes.length == 1 ? dom.blockParentOrBody(nodes[0]) : dom.commonAncestor.apply(null, nodes);

        if (dom.isInline(commonAncestor))
            commonAncestor = dom.blockParentOrBody(commonAncestor);
            
        var ancestors = dom.significantChildNodes(commonAncestor);

        var position = findNodeIndex(ancestors[0]);

        var wrapper = dom.create(commonAncestor.ownerDocument, tag, attributes);
            
        for (var i = 0; i < ancestors.length; i++) {
            var ancestor = ancestors[i];
            if (dom.isBlock(ancestor)){
                dom.attr(ancestor, attributes);

                if (wrapper.childNodes.length) {
                    dom.insertBefore(wrapper, ancestor);
                    wrapper = wrapper.cloneNode(false);
                }

                position = findNodeIndex(ancestor) + 1;

                continue;
            }
                
            wrapper.appendChild(ancestor);
        }

        if (wrapper.firstChild)
            dom.insertAt(commonAncestor, wrapper, position);
    }

    this.apply = function (nodes) {
        var formatNode = dom.is(nodes[0], 'img') ? nodes[0] : finder.findSuitable(nodes);

        var formatToApply = formatNode ? formatByName(dom.name(formatNode), format) : format[0];

        var tag = formatToApply.tags[0];
        var attributes = $.extend({}, formatToApply.attr, values);

        if (formatNode)
            dom.attr(formatNode, attributes);
        else
            wrap(tag, attributes, nodes);
    }

    this.remove = function (nodes) {
        for (var i = 0, l = nodes.length; i < l; i++) {
            var formatNode = finder.findFormat(nodes[i]);
            if (formatNode)
                if (dom.ofType(formatNode, ['p', 'img', 'li']))
                    dom.unstyle(formatNode, formatByName(dom.name(formatNode), format).attr.style);
                else
                    dom.unwrap(formatNode);
        }
    }

    this.toggle = function (range) {
        var nodes = textNodes(range);
        if (!nodes.length) {
            range.selectNodeContents(range.commonAncestorContainer);
            nodes = textNodes(range);
            if (!nodes.length)
                nodes = dom.significantChildNodes(range.commonAncestorContainer);
        }
        
        if (finder.isFormatted(nodes))
            this.remove(nodes);
        else
            this.apply(nodes);
    }
}

function GreedyBlockFormatter(format, values)
{
    var finder = new BlockFormatFinder(format);

    this.apply = function(nodes) {
        var blocks = blockParents(nodes);
        var formatTag = format[0].tags[0];

        for (var i = 0, len = blocks.length; i < len; i++) {
            if (dom.is(blocks[i], 'li')) {
                var list = blocks[i].parentNode;
                var formatter = new ListFormatter(list.nodeName.toLowerCase(), formatTag);
                var range = this.editor.createRange();
                range.selectNode(blocks[i]);
                formatter.toggle(range);
            } else {
                var newNode = dom.create(blocks[i].ownerDocument, formatTag);
                dom.insertBefore(newNode, blocks[i]);
                newNode.appendChild(blocks[i]);
                dom.unwrap(blocks[i]);
            }
        }
    }

    this.toggle = function(range) {
        var nodes = textNodes(range);
        if (!nodes.length) {
            range.selectNodeContents(range.commonAncestorContainer);
            nodes = textNodes(range);
            if (!nodes.length)
                nodes = dom.significantChildNodes(range.commonAncestorContainer);
        }

        this.apply(nodes);
    }
}

function FormatCommand(options) {
    Command.call(this, options.range);
    var formatter = options.formatter();

    this.exec = function () {
        var range = this.lockRange(true);
        
        formatter.editor = this.editor;
            
        formatter.toggle(range);

        this.releaseRange(range)
    }
};