﻿(function ($) {

    var $t = $.telerik;

    $t.autocomplete = function (element, options) {
        this.element = element;
        this.$element = $(element);
        this.$text = this.$element;

        $.extend(this, options);

        $(document).bind('mousedown', $.proxy(function (e) {
            var $dropDown = this.$dropDown;
            var isDropDown = $dropDown && $dropDown.parent().length > 0;

            if (isDropDown && !$.contains(this.element, e.target) && !$.contains($dropDown.parent()[0], e.target))
                $t.dropDownList.close(this);

        }, this));

        this.$element
            .bind({
                focus: $.proxy(function (e) { e.stopPropagation(); }, this),
                change: $.proxy(function (e) { if (this.onChange) this.onChange(e); }, this),
                keydown: $.proxy(this.keydown, this),
                keypress: $.proxy(function (e) {
                    var key = e.keyCode || e.charCode;

                    if (key == 0 || $.inArray(key, $t.dropDownList.keycodes) != -1) return true;

                    this.resetTimer(); //reset and start filtering after delay

                }, this),
                selectOnDataBinding: $.proxy(function (e) {
                    var $item = this.highlightFirst ? this.$items.first() : null;
                    if ($item) $item.addClass('t-state-selected');
                }, this)
            });

        $t.bind(this, {
            dataBinding: this.onDataBinding,
            dataBound: this.onDataBound,
            error: this.onError,
            open: this.onOpen,
            close: this.onClose,
            load: this.onLoad
        });
    }

    $.extend($t.autocomplete, {

        filter: function (component) {
            component.isFiltered = true;

            var performAjax = true;
            var data = component.data;
            var input = component.$text[0];
            var text = input.value;

            if (component.multiple) { //only for the AutoComplete
                text = text.split(component.separator);
                text = text[$t.combobox.valueArrayIndex(input, component.separator)];
            }

            if (text.length < component.minChars) return;

            var filterIndex = component.filter;
            if (component.isAjax()) {

                performAjax = true;

                if (component.cache && data && data.length > 0) {

                    if (!component.$dropDown) component.dropDown();

                    component.filters[filterIndex](component, data, text);

                    var filteredDataIndexes = component.filteredDataIndexes;

                    if (filteredDataIndexes && filteredDataIndexes.length > 0) performAjax = false;
                }

                if (performAjax) {

                    var postData = {};
                    postData[component.queryString.text] = text;

                    component.ajaxRequest(function (data) {

                        if (data && data.length == 0) {
                            $t.dropDownList.close(component);
                            component.dataBind();
                            return;
                        }

                        if (!component.$dropDown) {
                            $t.dropDownList.close(component);
                            component.dataBind();
                        }

                        component.data = data;

                        $t.trigger(component.element, 'dataBound');

                        component.filters[filterIndex](component, data, text);

                        var $items = component.$items;
                        var itemsLength = $items.length;

                        component.$dropDown.css('height', itemsLength > 10 ? component.dropDownHeight : 'auto');

                        if (itemsLength > 0) {
                            if (!component.isOpened()) $t.dropDownList.open(component);
                            $t.autocomplete.autoFill(component, $items.first().text());
                        }
                        else $t.dropDownList.close(component);

                    }, { data: postData });
                }
            } else {
                performAjax = false;

                if (!component.$dropDown) component.dropDown();

                component.filters[filterIndex](component, component.data, text);
            }

            if (!performAjax) {
                var $items = component.$items;
                var itemsLength = $items.length;
                var selectedIndex = component.selectedIndex;

                component.$dropDown.css('height', itemsLength > 10 ? component.dropDownHeight : 'auto');

                var itemText = filterIndex == 0
                     ? selectedIndex != -1
                     ? $items[selectedIndex].innerText || $items[selectedIndex].textContent
                     : ''
                     : $items.length > 0
                     ? $items.first().text()
                     : '';

                $t.autocomplete.autoFill(component, itemText);

                if (itemsLength == 0) $t.dropDownList.close(component);
                else if (!component.isOpened()) $t.dropDownList.open(component);
            }
        },

        autoFill: function (component, itemText) {
            if (component.autoFill && (component.lastKeyCode != 8 && component.lastKeyCode != 46)) {

                var input = component.$text[0];
                var textBoxValue = input.value;
                var separator = component.separator;

                var endIndex = $t.caretPos(input);

                var lastSeparatorIndex = separator ? $t.lastIndexOf(textBoxValue.substring(0, endIndex), separator) : -1;

                var startIndex = lastSeparatorIndex != -1 ? lastSeparatorIndex + separator.length : 0;

                var filterString = textBoxValue.substring(startIndex, endIndex);

                var matchIndex = itemText.toLowerCase().indexOf(filterString.toLowerCase());

                if (matchIndex != -1) {

                    var stringToAppend = itemText.substring(matchIndex + filterString.length);
                    var wordIndex = $t.combobox.valueArrayIndex(input, separator);
                    var split = textBoxValue.split(separator);
                    split[wordIndex] = filterString + stringToAppend;
                    input.value = split.join(separator) + (component.multiple && wordIndex != 0 && wordIndex == split.length - 1 ? separator : '');

                    $t.combobox.selection(input, endIndex, endIndex + stringToAppend.length);
                }
            }
        }
    });

    $t.autocomplete.prototype = $.extend(new $t.combobox(), {

        ensureData: function (callback) {
            var minChars = this.minChars;
            var textValue = this.$text.val();
            var textValueLength = textValue.length;

            if (this.isAjax() && textValueLength >= minChars) {

                var data = {};
                data[this.queryString.text] = textValue;

                this.ajaxRequest(function (data) {

                    this.dataBind(data);

                    $t.trigger(this.element, 'dataBound');

                    if (callback) callback(this);
                },
                { data: data });
            }

            if (!this.data) this.data = [];
        },

        resetTimer: function () {
            clearTimeout(this.timeout);
            this.timeout = setTimeout($.proxy(function () { $t.autocomplete.filter(this) }, this), this.delay);
        },

        filters: [
            $t.combobox.firstMatch,

            createItemsFilter(false, function (inputText, itemText) {
                return itemText.slice(0, inputText.length).toLowerCase() == inputText.toLowerCase();
            }),

            createItemsFilter(true, function (inputText, itemText) {
                return itemText && itemText.toLowerCase().indexOf(inputText.toLowerCase()) != -1
            })
        ],

        select: function (item) {

            var index = this.highlight(item);

            if (index == -1) return;

            var filteredDataIndexes = this.filteredDataIndexes;
            var itemIndex = (filteredDataIndexes && filteredDataIndexes.length) > 0 ? filteredDataIndexes[index] : index;

            var dataText = this.data[itemIndex];

            var value = dataText;

            if (this.multiple) {

                var $element = this.$element
                var separator = this.separator;
                var wordIndex = $t.combobox.valueArrayIndex($element[0], separator);

                value = $element.val().split(separator);
                value[wordIndex] = dataText;
                value = value.join(separator) + (wordIndex == value.length - 1 ? separator : '');
            }

            this.text(value);
        },

        value: function () {
            return this.text.apply(this, arguments);
        },

        showBusy: function () {
            this.busyTimeout = setTimeout($.proxy(function () {
                this.$element.addClass('t-loading');
            }, this), 100);
        },

        hideBusy: function () {
            clearTimeout(this.busyTimeout);
            this.$element.removeClass('t-loading');
        },

        keydown: function (e) {

            var key = e.keyCode || e.which;
            this.lastKeyCode = key;

            if (!e.shiftKey && key > 36 && key < 41 && key != 37 && key != 39) {

                e.preventDefault();

                if (this.isOpened()) {

                    if (!this.$items) this.dropDown(); //creates dropdown

                    var $items = this.$items;

                    var $selectedItem = $items.filter('.t-state-selected:first');

                    var $item = [];

                    if (key == 38) {
                        var prevItem = $selectedItem.prev();
                        $item = prevItem.length ? prevItem : $items.last();
                    } else if (key == 40) {
                        var nextItem = $selectedItem.next();
                        $item = nextItem.length ? nextItem : $items.first();
                    }

                    if ($item.length) {

                        var item = $item[0];
                        this.highlight(item);
                        this.scrollTo(item);

                        $t.autocomplete.autoFill(this, $item.text());
                    }
                }
            }

            if (key == 8 || key == 46) {
                var $element = this.$element;

                if ($element.val() != '') this.resetTimer(); //reset and start filtering after delay

                setTimeout($.proxy(function () {
                    if ($element.val() == '') {
                        $t.dropDownList.close(this);
                    }
                }, this), 0);
            }

            if (key == 13) { //move caret to the end of the input
                if (this.$items.length) {
                    var $selectedItems = this.$items.filter('.t-state-selected:first');

                    if (this.isOpened()) e.preventDefault();

                    if ($selectedItems.length > 0) {
                        this.select($selectedItems[0]);
                        $t.dropDownList.close(this);
                        $t.combobox.moveToEnd(this.element);
                    }
                }
            }

            if (key == 27 || key == 9)
                $t.dropDownList.close(this);
        }
    });

    function createItemsFilter(global, condition) {
        return function (component, data, inputText) {
            if (!data || data.length == 0) return;

            var filteredDataIndexes = $.map(data, function (item, index) {
                if (condition(inputText, item.Text || item)) return index;
            });

            var filteredDataIndexesLength = filteredDataIndexes.length;

            var formatRegExp = new RegExp("(?![^&;]+;)(?!<[^<>]*)(" + inputText.replace(/([\^\$\(\)\[\]\{\}\*\.\+\?\|\\])/gi, "\\$1") + ")(?![^<>]*>)(?![^&;]+;)", global ? 'ig' : 'i');

            var $items = filteredDataIndexesLength > 0 ? $($t.dropDownList.itemsHtml(component, data, filteredDataIndexes, formatRegExp)) : filteredDataIndexes /* filteredDataIndexes is [] */;
            component.$dropDown.find('> ul').html($items);
            component.filteredDataIndexes = filteredDataIndexes;
            component.$items = $items;

            component.selectedIndex = -1;

            if (filteredDataIndexesLength > 0) {
                $items.removeClass('t-state-selected');
                $t.combobox.highlightFirstOnFilter(component, $items);
            }
        };
    }

    // jQuery extender
    $.fn.tAutoComplete = function (options) {
        return $t.create(this, {
            name: 'tAutoComplete',
            init: function (element, options) {
                return new $t.autocomplete(element, options)
            },
            options: options
        });
    };

    // default options
    $.fn.tAutoComplete.defaults = {
        effects: $t.fx.slide.defaults(),
        filter: 1,
        delay: 200,
        minChars: 1,
        cache: true,
        autoFill: false,
        highlightFirst: false,
        queryString: {
            text: 'text'
        },
        multiple: false,
        separator: ', '
    };
})(jQuery);