﻿(function ($) {

    var $t = $.telerik;

    $.extend($t.combobox, {

        filter: function (component) {
            component.isFiltered = true;

            var performAjax = true;
            var data = component.data;
            var input = component.$text[0];
            var text = input.value;

            if (text.length < component.minChars) return;

            var filterIndex = component.filter;
            if (component.isAjax()) {

                performAjax = true;

                if (component.cache && data && data.length > 0) {

                    if (!component.$dropDown) component.dropDown();

                    component.filters[filterIndex](component, data, text);

                    var filteredDataIndexes = component.filteredDataIndexes;

                    if (filteredDataIndexes && filteredDataIndexes.length > 0) performAjax = false;
                }

                if (performAjax) {

                    var postData = {};
                    postData[component.queryString.text] = text;

                    component.ajaxRequest(function (data) {

                        if (data && data.length == 0) {
                            $t.dropDownList.close(component);
                            component.dataBind();
                            return;
                        }

                        if (!component.$dropDown) {
                            $t.dropDownList.close(component);
                            component.dataBind();
                        }

                        component.data = data;

                        $t.trigger(component.element, 'dataBound');

                        component.filters[filterIndex](component, data, text);

                        var $items = component.$items;
                        var itemsLength = $items.length;

                        component.$dropDown.css('height', itemsLength > 10 ? component.dropDownHeight : 'auto');

                        if (itemsLength > 0) {
                            if (!component.isOpened()) $t.dropDownList.open(component);
                            $t.combobox.autoFill(component, $items.first().text());
                        }
                        else $t.dropDownList.close(component);

                    }, { data: postData });
                }
            } else {
                performAjax = false;

                if (!component.$dropDown) component.dropDown();

                component.filters[filterIndex](component, component.data, text);
            }

            if (!performAjax) {
                var $items = component.$items;
                var itemsLength = $items.length;
                var selectedIndex = component.selectedIndex;

                component.$dropDown.css('height', itemsLength > 10 ? component.dropDownHeight : 'auto');

                var itemText = filterIndex == 0
                     ? selectedIndex != -1
                     ? $items[selectedIndex].innerText || $items[selectedIndex].textContent
                     : ''
                     : $items.length > 0
                     ? $items.first().text()
                     : '';

                $t.combobox.autoFill(component, itemText);

                if (itemsLength == 0) $t.dropDownList.close(component);
                else if (!component.isOpened()) $t.dropDownList.open(component);
            }
        }
    });

    $.extend($t.combobox.prototype, {

        ensureData: function (callback) {
            var minChars = this.minChars;
            var textValue = this.$text.val();
            var textValueLength = textValue.length;

            if (this.isAjax() && textValueLength >= minChars) {

                var data = {};
                data[this.queryString.text] = textValue;

                this.ajaxRequest(function (data) {

                    this.dataBind(data);

                    $t.trigger(this.element, 'dataBound');

                    if (callback) callback(this);
                },
                { data: data });
            }

            if (!this.data) this.data = [];
        },

        resetTimer: function () {
            clearTimeout(this.timeout);
            this.timeout = setTimeout($.proxy(function () { $t.combobox.filter(this) }, this), this.delay);
        },

        filters: [
            $t.combobox.firstMatch,

            createItemsFilter(false, function (inputText, itemText) {
                return itemText.slice(0, inputText.length).toLowerCase() == inputText.toLowerCase();
            }),

            createItemsFilter(true, function (inputText, itemText) {
                return itemText && itemText.toLowerCase().indexOf(inputText.toLowerCase()) != -1
            })
        ]
    });

    function createItemsFilter(global, condition) {
        return function (component, data, inputText) {
            if (!data || data.length == 0) return;

            var filteredDataIndexes = $.map(data, function (item, index) {
                if (condition(inputText, item.Text || item)) return index;
            });

            var filteredDataIndexesLength = filteredDataIndexes.length;

            var formatRegExp = new RegExp("(?![^&;]+;)(?!<[^<>]*)(" + inputText.replace(/([\^\$\(\)\[\]\{\}\*\.\+\?\|\\])/gi, "\\$1") + ")(?![^<>]*>)(?![^&;]+;)", global ? 'ig' : 'i');

            var $items = filteredDataIndexesLength > 0 ? $($t.dropDownList.itemsHtml(component, data, filteredDataIndexes, formatRegExp)) : filteredDataIndexes /* filteredDataIndexes is [] */;
            component.$dropDown.find('> ul').html($items);
            component.filteredDataIndexes = filteredDataIndexes;
            component.$items = $items;

            component.selectedIndex = -1;

            if (filteredDataIndexesLength > 0) {
                $items.removeClass('t-state-selected');
                $t.combobox.highlightFirstOnFilter(component, $items);
            }
        };
    }

    // default options
    $.extend($.fn.tComboBox.defaults, {
        filter: 0,
        delay: 200,
        minChars: 0,
        cache: true,
        queryString: {
            text: 'text'
        }
    });
})(jQuery);