﻿(function ($) {

    var error;

    var $t = $.telerik;

    function firstMatch(data, $items, text) {
        if (!data || !$items) return null;

        var valueLength = text.length;
        text = text.toLowerCase();

        for (var i = 0, length = data.length; i < length; i++) {
            if (data[i].Text.slice(0, valueLength).toLowerCase() == text)
                return $items[i];
        }
    }

    $t.dropDownList = function (element, options) {

        $.extend(this, options);

        this.element = element;
        this.$element = $(element);
        this.$text = this.$element.find('> .t-dropdown-wrap > .t-input');
        this.cachedInput = '';

        //allow element to be focused
        if (!this.$element.attr('tabIndex')) this.$element.attr('tabIndex', 0);

        $t.dropDownList.initialize(this);

        this.$element
            .bind({
                keydown: $.proxy(this.keydown, this),
                keypress: $.proxy(function (e) {
                    var key = e.keyCode || e.charCode;

                    if (key == 0 || $.inArray(key, $t.dropDownList.keycodes) != -1 || e.ctrlKey || e.altKey || e.shiftKey) return;

                    if (!this.$items) {
                        this.dropDown();
                        return;
                    }

                    var cachedInput = this.cachedInput;

                    cachedInput += String.fromCharCode(key)

                    if (cachedInput) {

                        var item = firstMatch(this.data, this.$items, cachedInput);

                        if (item) {
                            this.select(item);
                            this.scrollTo(item);
                        }

                        this.cachedInput = cachedInput;
                    }

                    this.resetTimer();
                }, this),
                click: $.proxy(function (e) {
                    this.$element.focus();

                    if (!this.isOpened()) {
                        if (!this.$dropDown) this.dropDown($t.dropDownList.open);
                        else $t.dropDownList.open(this);
                    } else {
                        $t.dropDownList.close(this);
                    }
                }, this),
                selectOnDataBinding: $.proxy(function (e) {
                    var selectedIndex = e.selectedIndex;
                    var selectedItemsList = e.selectedItemsList;

                    var $items = this.$items;
                    var selectedItemsLength = selectedItemsList.length;
                    var item = selectedIndex != -1 && selectedIndex < $items.length
                         ? $items[selectedIndex]
                         : selectedItemsLength > 0
                         ? selectedItemsList[selectedItemsLength - 1]
                         : $items[0];

                    this.select(item);
                    this.previousValue = this.value();
                }, this)
            });
    }

    $t.dropDownList.prototype = {
        dropDown: function (callback) {
            if (!this.$dropDown && !error) {
                this.ensureData(callback);

                if (!this.isAjax()) {
                    this.dataBind(this.data);
                    if (callback) callback(this);
                }
            }

            return this.$dropDown;
        },

        ensureData: function (callback) {

            if (this.isAjax()) {
                this.ajaxRequest(function (data) {

                    this.dataBind(data);

                    $t.trigger(this.element, 'dataBound');

                    if (callback) callback(this);
                });
            }

            if (!this.data) this.data = [];
        },

        isAjax: function () {
            return this.ajax || this.ws || this.onDataBinding;
        },

        ajaxOptions: function (complete, options) {

            var result = {
                url: (this.ajax || this.ws)['selectUrl'],
                type: 'POST',
                data: {},
                dataType: 'text', // using 'text' instead of 'json' because of DateTime serialization
                error: $.proxy(function (xhr, status) {
                    error = true;
                    if ($t.ajaxError(this.element, 'error', xhr, status))
                        return;
                }, this),
                complete: $.proxy(this.hideBusy, this),

                success: $.proxy(function (data, status, xhr) {
                    try {
                        data = eval('(' + data + ')');
                    } catch (e) {
                        // in case the result is not JSON raise the 'error' event
                        if (!$t.ajaxError(this.element, 'error', xhr, 'parseeror'))
                            alert('Error! The requested URL did not return JSON.');
                        error = true;
                        return;
                    }
                    data = data.d || data; // Support the `d` returned by MS Web Services 

                    if (complete)
                        complete.call(this, data);

                }, this)
            }

            $.extend(result, options);

            if (this.ws) {
                result.data = $t.toJson(result.data);
                result.contentType = 'application/json; charset=utf-8';
            }

            return result;
        },

        ajaxRequest: function (complete, options) {
            var e = {};

            if ($t.trigger(this.element, 'dataBinding', e))
                return;

            if (this.ajax || this.ws) {
                this.showBusy();
                $.ajax(this.ajaxOptions(complete, { data: $.extend({}, options ? options.data : {}, e.data) }));
            }
            else
                if (complete) complete.call(this, this.data);
        },

        showBusy: function () {
            this.busyTimeout = setTimeout($.proxy(function () {
                this.$element.find('> .t-dropdown-wrap .t-icon').addClass('t-loading');
            }, this), 100);
        },

        hideBusy: function () {
            clearTimeout(this.busyTimeout);
            this.$element.find('> .t-dropdown-wrap .t-icon').removeClass('t-loading');
        },

        open: function () {
            if (this.data.length == 0 || this.isOpened()) return;

            var $dropDown = this.dropDown();
            if (!$dropDown) {
                this.dropDown($t.dropDownList.open);
                return;
            }

            var selector = '.t-reset > .t-item';
            var $element = this.$element;

            $t.trigger(this.element, 'beforeOpen');

            $dropDown.delegate(selector, 'mouseenter', $t.hover)
                         .delegate(selector, 'mouseleave', $t.leave)
                         .delegate(selector, 'click',
                                   $.proxy(function (e) {
                                       this.select($(e.target).closest('.t-item')[0]);
                                       $t.dropDownList.change(this);
                                       $t.dropDownList.close(this);
                                   }, this))
                         .appendTo(document.body);

            // reposition
            var elementPosition = $element.offset();

            elementPosition.top += $element.outerHeight();

            $t.fx._wrap($dropDown).css($.extend({
                position: 'absolute',
                zIndex: $element.css('zIndex')
            }, elementPosition));

            // animate
            $t.fx.play(this.effects, $dropDown, { direction: 'bottom' }, $.proxy(function () {
                var selectedIndex = this.selectedIndex;
                if ((selectedIndex || selectedIndex == 0) && selectedIndex != -1) {
                    this.scrollTo(this.$items[selectedIndex]);
                }
            }, this));
        },

        close: function () {
            var $dropDown = this.dropDown();
            $t.fx.rewind(this.effects, $dropDown, { direction: 'bottom' }, function () {
                $dropDown.parent().remove();
            });
        },

        highlight: function (item) {

            var $items = this.$items || this.dropDown().find('> .t-reset > .t-item');

            var index = -1;

            var rebind = function (component) {
                var previousValue = component.previousValue;
                component.close();
                component.dataBind(component.data);
                component.previousValue = previousValue;
                component.$items.removeClass('t-state-selected');
                $(component.$items[index]).addClass('t-state-selected');
            };

            if (!isNaN(item - 0) && item > -1 && item < $items.length) {

                index = item;

                rebind(this);

            } else if ($.isFunction(item)) {

                for (var i = 0, len = this.data.length; i < len; i++) {
                    if (item(this.data[i]))
                        index = i;
                }

                if (index != -1) rebind(this);

            } else {
                $items.removeClass('t-state-selected');
                index = $(item).addClass('t-state-selected').index();
            }

            return index;
        },

        select: function (item) {

            var index = this.highlight(item);

            if (index == -1) return;

            // set selected Index
            this.selectedIndex = index;

            $t.dropDownList.updateTextAndValue(this, $(this.$items[index]).text(), this.data[index].Value);
        },

        text: function (text) {
            return this.$text.html(text);
        },

        value: function (value) {
            return this.$input.val(value);
        },

        dataBind: function (data) {
            this.data = data || [];

            var attr = this.dropDownAttr || '';
            var $dropDown = $t.dropDownList.dropDownHtml(this, this.data, attr).hide();

            // cache LI items
            var $items = $dropDown.find('.t-reset > .t-item');
            this.$items = $items;

            if (attr.indexOf('width:') == -1) $dropDown.css('width', this.$element.outerWidth() - 2);
            if (attr.indexOf('overflow-y:') == -1) $dropDown.css('overflow-y', 'auto');
            this.dropDownHeight = attr.indexOf('height:') == -1 ? 200 : $dropDown.css('height');
            $dropDown.css('height', $items.length > 10 ? this.dropDownHeight : 'auto')

            if ($items.length > 0)
                $t.trigger(this.element, 'selectOnDataBinding', {
                    selectedIndex: this.index,
                    selectedItemsList: $items.filter('.t-state-selected')
                });

            this.$dropDown = $dropDown;
        },

        resetTimer: function () {
            clearTimeout(this.timeout);
            this.timeout = setTimeout($.proxy(function () { this.cachedInput = '' }, this), 1000);
        },

        keydown: function (e) {

            var key = e.keyCode || e.which;

            // close dropdown
            if (e.altKey && key == 38) {
                $t.dropDownList.close(this);
                return;
            }

            // open dropdown
            if (e.altKey && key == 40) {
                $t.dropDownList.open(this);
                return;
            }

            if (key > 34 && key < 41) {

                e.preventDefault();

                if (!this.$items) {
                    this.dropDown(); // creates dropdown
                    return;
                }
                var $items = this.$items;
                var $selectedItem = $($items[this.selectedIndex]);

                var $item = (key == 35) ? $items.last() // end
                         : (key == 36) ? $items.first() // home
                         : (key == 37 || key == 38) ? $selectedItem.prev() // up
                         : (key == 39 || key == 40) ? $selectedItem.next() // down
                         : [];

                if ($item.length) {
                    var item = $item[0];
                    this.select(item);
                    this.scrollTo(item);
                    if (!this.isOpened()) $t.dropDownList.change(this);
                }
            }

            if (key == 8) {
                this.resetTimer();
                e.preventDefault();
                this.cachedInput = this.cachedInput.slice(0, -1);
            }

            if (key == 9 || key == 13 || key == 27) {
                $t.dropDownList.change(this);
                $t.dropDownList.close(this);
            }
        },

        isOpened: function () {
            var $dropDown = this.$dropDown;
            return $dropDown && $dropDown.is(':visible');
        },

        scrollTo: function (item) {

            if (!item) return;

            var itemOffsetTop = item.offsetTop;
            var itemOffsetHeight = item.offsetHeight;

            var dropDown = this.dropDown()[0];
            var dropDownScrollTop = dropDown.scrollTop;
            var dropDownOffsetHeight = dropDown.clientHeight;
            var bottomDistance = itemOffsetTop + itemOffsetHeight;

            dropDown.scrollTop = dropDownScrollTop > itemOffsetTop
                                 ? itemOffsetTop
                                 : bottomDistance > (dropDownScrollTop + dropDownOffsetHeight)
                                 ? bottomDistance - dropDownOffsetHeight
                                 : dropDownScrollTop;
        }
    };

    $.extend($t.dropDownList, {
        initialize: function (component) {
            var $element = component.$element;

            component.$input = $element.find('input:last');

            //set selected item text
            var data = component.data;
            var length = data ? data.length : 0;
            var selectedIndex = component.index;

            if (!component.isAjax() && length > 0 && selectedIndex < length && selectedIndex != -1) {
                var dataItem = data[selectedIndex];
                $t.dropDownList.updateTextAndValue(component, dataItem.Text, dataItem.Value);
            }

            $t.bind(component, {
                dataBinding: component.onDataBinding,
                dataBound: component.onDataBound,
                error: component.onError,
                open: component.onOpen,
                close: component.onClose,
                change: component.onChange,
                load: component.onLoad
            });

            $(document).bind('mousedown', $.proxy(function (e) {
                var $dropDown = component.$dropDown;
                var isDropDown = $dropDown && $dropDown.parent().length > 0;

                if (isDropDown && !$.contains(component.element, e.target) && !$.contains($dropDown.parent()[0], e.target)) {
                    $t.dropDownList.change(component);
                    $t.dropDownList.close(component);
                }

            }, component));
        },

        dropDownHtml: function (component, items, attr) {

            return $(new $t.stringBuilder().cat('<div ')
                             .catIf(attr, attr)
                             .cat(' class="t-popup t-group"><ul class="t-reset">')
                             .cat($t.dropDownList.itemsHtml(component, items, []))
                             .cat('</ul></div>')
                             .string());
        },

        itemsHtml: function (component, items, indexes, regExp) {
            var html = new $t.stringBuilder();
            if (items) {
                var isFilteredItems = indexes.length > 0;

                var length = isFilteredItems ? indexes.length : items.length;

                for (var i = 0; i < length; i++) {

                    var item = items[isFilteredItems ? indexes[i] : i];
                    var itemText = item.Text || item;

                    var e = {
                        text: regExp ? itemText.replace(regExp, "<strong>$1</strong>") : itemText,
                        dataItem: item
                    };

                    if (component.onItemDataBound) component.onItemDataBound.call(component.element, e);

                    html.cat('<li class="t-item">').cat(e.text).cat('</li>');
                }
            }
            return html.string();
        },

        updateTextAndValue: function (component, text, value) {
            component.text(text);
            component.value((value || value == 0) ? value : text);
        },

        change: function (component) {
            var value = component.value();
            var previousValue = component.previousValue;

            if (previousValue == undefined || value != previousValue)
                $t.trigger(component.element, 'change', { value: value });

            component.previousValue = value;
        },

        open: function (component) {
            if ((component.$items && component.$items.length > 0) && !component.isOpened() && !$t.trigger(component.element, 'open')) $.proxy(component.open(), component);
        },

        close: function (component) {
            if (component.isOpened() && !$t.trigger(component.element, 'close')) component.close();
        },

        keycodes: [8, // backspace
                   9, // tab
                  13, // enter
                  27, // escape
                  37, // left arrow
                  38, // up arrow
                  39, // right arrow
                  40, // down arrow
                  35, // end
                  36, // home
                  46] // delete
    });

    $.fn.tDropDownList = function (options) {
        return $t.create(this, {
            name: 'tDropDownList',
            init: function (element, options) {
                return new $t.dropDownList(element, options)
            },
            options: options
        });
    };

    // default options
    $.fn.tDropDownList.defaults = {
        effects: $t.fx.slide.defaults(),
        index: 0
    };

    // combobox base
    $t.combobox = function (element, options) {
        this.element = element;
        this.$element = $(element);
        this.$text = this.$element.find('> .t-dropdown-wrap > .t-input').attr('autocomplete', 'off');

        $.extend(this, options);

        $t.dropDownList.initialize(this);

        this.$text
            .bind({
                change: $.proxy(function (e) { e.stopPropagation(); }, this),
                keydown: $.proxy(this.keydown, this),
                keypress: $.proxy(function (e) {

                    var key = e.keyCode || e.charCode;

                    if (key == 0 || $.inArray(key, $t.dropDownList.keycodes) != -1) return true;

                    // always set value. Select(item) will override it if needed.
                    this.value(this.$text.val() + String.fromCharCode(key));

                    if (this.filters) {
                        this.resetTimer(); // reset and start filtering after delay
                    } else {
                        setTimeout($.proxy(function () {

                            $t.combobox.firstMatch(this, this.data, this.$text[0].value);

                            var $items = this.$items;
                            var selectedIndex = this.selectedIndex;

                            var itemText = selectedIndex != -1
                                        ? $items[selectedIndex].innerText || $items[selectedIndex].textContent
                                        : '';

                            $t.combobox.autoFill(this, itemText)
                        }, this), 0);
                    }
                }, this),
                focus: $.proxy(function (e) {
                    if (!this.$dropDown)
                        this.dropDown($t.dropDownList.open);
                    else
                        $t.dropDownList.open(this);

                    //select input text.
                    var $text = this.$text;
                    $t.combobox.selection($text[0], 0, $text.val().length);

                }, this)
            });

        this.$element
            .bind({
                beforeOpen: $.proxy(function () {
                    var data = this.data;
                    if (data.length == 0) return;

                    var text = this.$text.val();
                    var selectedIndex = this.selectedIndex;

                    if (selectedIndex != -1 && this.isFiltered) {
                        if (text == data[selectedIndex].Text) {
                            this.filteredDataIndexes = [];

                            var $dropDown = this.$dropDown;
                            var $items = $($t.dropDownList.itemsHtml(this, this.data, []));
                            $dropDown.find('> ul').html($items);
                            $dropDown.css('height', $items.length > 10 ? this.dropDownHeight : 'auto');
                            this.$items = $items;

                            this.select($items[selectedIndex]);
                        } else
                            this.filters[this.filter](this, this.data, text);

                        this.isFiltered = false;
                    }
                }, this),
                selectOnDataBinding: $.proxy(function (e) {
                    var $items = this.$items;
                    var selectedIndex = e.selectedIndex;
                    var selectedItemsList = e.selectedItemsList;
                    var selectedItemsLength = selectedItemsList.length;

                    var item = selectedIndex != -1 && selectedIndex < $items.length
                       ? $items[selectedIndex]
                       : selectedItemsLength > 0
                       ? selectedItemsList[selectedItemsLength - 1]
                       : null;

                    if (item)
                        this.select(item);
                    else if (this.highlightFirst)
                        this.highlight($items[0]);

                    this.previousValue = this.value();
                }, this)
            })
            .find('> .t-dropdown-wrap > .t-select')
            .click($.proxy(function (e) {
                error = false;
                if (!this.isOpened())
                    this.$text[0].focus();
                else
                    $t.dropDownList.close(this);
            }, this));
    }

    $.extend($t.combobox, {
        moveToEnd: function (element) {
            if (element.createTextRange) {
                var range = element.createTextRange();
                range.moveStart('textedit', 1);
                range.select();
            }
        },

        valueArrayIndex: function (input, separator) {
            return input.value.substring(0, $t.caretPos(input)).split(separator).length - 1;
        },

        selection: function (input, start, end) {
            if (input.createTextRange) {
                var selRange = input.createTextRange();
                selRange.collapse(true);
                selRange.moveStart('character', start);
                selRange.moveEnd('character', end - start);
                selRange.select();
            } else if (input.selectionStart) {
                input.selectionStart = start;
                input.selectionEnd = end;
            }
        },

        autoFill: function (component, itemText) {
            if (component.autoFill && (component.lastKeyCode != 8 && component.lastKeyCode != 46)) {

                var input = component.$text[0];

                var endIndex = $t.caretPos(input);

                var filterString = input.value.substring(0, endIndex);

                var matchIndex = itemText.toLowerCase().indexOf(filterString.toLowerCase());

                if (matchIndex != -1) {

                    var stringToAppend = itemText.substring(matchIndex + filterString.length);

                    input.value = filterString + stringToAppend;

                    $t.combobox.selection(input, endIndex, endIndex + stringToAppend.length);
                }
            }
        },

        firstMatch: function (component, data, inputText) {
            if (!data || data.length == 0) return;
            var $items = component.$items;

            if ($items.length == 0 || component.isAjax()) {
                $items = $($t.dropDownList.itemsHtml(component, data, []));
                component.$dropDown.find('> ul').html($items); //add items if they were not rendered yet
                component.$items = $items;
            }

            for (var i = 0, length = data.length; i < length; i++) {
                if (data[i].Text.slice(0, inputText.length).toLowerCase() == inputText.toLowerCase()) {
                    var $item = $($items[i]);

                    $items.removeClass('t-state-selected');
                    $item.addClass('t-state-selected');

                    component.selectedIndex = i;
                    component.scrollTo($items[i]);
                    return;
                }
            }

            $items.removeClass('t-state-selected');
            component.selectedIndex = -1;

            $t.combobox.highlightFirstOnFilter(component, $items);
        },

        highlightFirstOnFilter: function (component, $items) {
            if (component.highlightFirst) {
                $items.first().addClass('t-state-selected');
                component.scrollTo($items[0]);
            }
        }
    });

    $t.combobox.prototype = $.extend({}, new $t.dropDownList(), {
        text: function (text) {
            return this.$text.val(text);
        },

        select: function (item) {

            var index = this.highlight(item);

            if (index == -1) return;

            var filteredDataIndexes = this.filteredDataIndexes;

            //set selected Index
            this.selectedIndex = (filteredDataIndexes && filteredDataIndexes.length) > 0 ? filteredDataIndexes[index] : index;

            $t.dropDownList.updateTextAndValue(this, $(this.$items[this.selectedIndex]).text(), this.data[this.selectedIndex].Value);
        },

        keydown: function (e) {
            var key = e.keyCode || e.which;
            this.lastKeyCode = key;

            //close dropdown
            if (e.altKey && key == 38) {
                $t.dropDownList.close(this);
                return;
            }

            //open dropdown
            if (e.altKey && key == 40) {
                $t.dropDownList.open(this);
                return;
            }

            if (!e.shiftKey && (key == 38 || key == 40)) {

                e.preventDefault();

                if (!this.$items) this.dropDown(); //creates dropdown

                var $items = this.$items;

                var $selectedItem = $items.filter('.t-state-selected:first');

                var $item = $selectedItem.length == 0 || $items.length == 1 ? $items.first()
                             : (key == 38) ? $selectedItem.prev() // up
                             : (key == 40) ? $selectedItem.next() // down
                             : [];

                if ($item.length) {
                    var item = $item[0];
                    this.select(item);
                    this.scrollTo(item);
                    if (!this.isOpened()) $t.dropDownList.change(this);
                }
            }

            if (key == 8 || key == 46) {
                var $text = this.$text;

                if ($text.val() != '') this.resetTimer(); //reset and start filtering after delay

                setTimeout($.proxy(function () {
                    if ($text.val() == '') {
                        this.selectedIndex = -1;
                        this.value('');
                    } else {
                        this.value(this.$text.val());
                    }
                }, this), 0);
            }

            if (key == 13) {
                var $selectedItems = this.$items.filter('.t-state-selected:first');

                if (this.isOpened()) e.preventDefault();

                if ($selectedItems.length > 0) {
                    this.select($selectedItems[0]);
                    $t.dropDownList.change(this);
                    $t.dropDownList.close(this);
                    $t.combobox.moveToEnd(this.$text[0]);
                }
            }

            if (key == 27 || key == 9) {
                $t.dropDownList.change(this);
                $t.dropDownList.close(this);
                if (key == 27) this.$text.blur();
            }
        }
    });

    $.fn.tComboBox = function (options) {
        return $t.create(this, {
            name: 'tComboBox',
            init: function (element, options) {
                return new $t.combobox(element, options)
            },
            options: options
        });
    };

    $.fn.tComboBox.defaults = {
        effects: $t.fx.slide.defaults(),
        index: -1,
        autoFill: true,
        highlightFirst: true
    };

})(jQuery);
